import json
from datetime import datetime
from test.test_const import CONST_BASE_URL, CONST_PORT, CONST_SSL
from unittest import TestCase

from pygrocydm.battery import BATTERIES_ENDPOINT, Battery
from pygrocydm.grocy_api_client import GrocyApiClient


class TestBattery(TestCase):

    def setUp(self):
        self.api = GrocyApiClient(CONST_BASE_URL, "demo_mode",  verify_ssl=CONST_SSL, port=CONST_PORT)
        self.endpoint = BATTERIES_ENDPOINT + '/1'

    def test_battery_data_diff_valid(self):
        battery = self.api.get_request(self.endpoint)
        battery_keys = battery.keys()
        moked_battery_json = """{
            "id": "1",
            "name": "Battery1",
            "description": "Warranty ends 2023",
            "used_in": "TV remote control",
            "charge_interval_days": "0",
            "row_created_timestamp": "2020-03-01 00:50:25"
        }"""
        moked_keys = json.loads(moked_battery_json).keys()
        self.assertCountEqual(list(battery_keys), list(moked_keys))

    def test_parse_json(self):
        battery = Battery(self.api.get_request(self.endpoint), self.api)
        assert isinstance(battery.id, int)
        assert isinstance(battery.description, str) or battery.description is None
        assert isinstance(battery.name, str)
        assert isinstance(battery.used_in, (str, None))
        assert isinstance(battery.charge_interval_days, int)
        assert isinstance(battery.row_created_timestamp, datetime)



import json
from datetime import datetime
from test.test_const import CONST_BASE_URL, CONST_PORT, CONST_SSL
from unittest import TestCase

from pygrocydm.grocy_api_client import GrocyApiClient
from pygrocydm.location import LOCATION_ENDPOINT, Location


class TestLocation(TestCase):

    def setUp(self):
        self.api = GrocyApiClient(CONST_BASE_URL, "demo_mode",  verify_ssl=CONST_SSL, port=CONST_PORT)
        self.endpoint = LOCATION_ENDPOINT + '/2'

    def test_location_data_diff_valid(self):
        location = self.api.get_request(self.endpoint)
        location_keys = location.keys()
        moked_location_json = """{
            "id": "2",
            "name": "Fridge",
            "description": null,
            "row_created_timestamp": "2020-03-01 00:50:24",
            "is_freezer": "0"
        }"""
        moked_keys = json.loads(moked_location_json).keys()
        self.assertCountEqual(list(location_keys), list(moked_keys))

    def test_parse_json(self):
        location = Location(self.api.get_request(self.endpoint), self.api)
        assert isinstance(location.id, int)
        assert isinstance(location.description, str) or location.description is None
        assert isinstance(location.name, str)
        assert isinstance(location.is_freezer, bool)
        assert isinstance(location.row_created_timestamp, datetime)



from datetime import datetime

from .grocy_api_client import GrocyApiClient, GrocyEntity
from .utils import parse_date, parse_int

BATTERIES_ENDPOINT = 'objects/batteries'


class Battery(GrocyEntity):
    def __init__(self, parsed_json, api: GrocyApiClient):
        self.__id = parse_int(parsed_json.get('id'))
        self.__name = parsed_json.get('name')
        self.__description = parsed_json.get('description', None)
        self.__used_in = parsed_json.get('used_in', None)
        self.__charge_interval_days = parse_int(
            parsed_json.get('charge_interval_days'),
            0)
        self.__row_created_timestamp = parse_date(
            parsed_json.get('row_created_timestamp'))
        self.__endpoint = '{}/{}'.format(BATTERIES_ENDPOINT, self.__id)
        super().__init__(api, self.__endpoint)

    @property
    def id(self) -> int:
        return self.__id

    @property
    def name(self) -> str:
        return self.__name

    @property
    def description(self) -> str:
        return self.__description

    @property
    def used_in(self) -> str:
        return self.__used_in

    @property
    def charge_interval_days(self) -> int:
        return self.__charge_interval_days

    @property
    def row_created_timestamp(self) -> datetime:
        return self.__row_created_timestamp



from datetime import datetime
from unittest import TestCase

import pygrocydm.utils as utils


class TestUtils(TestCase):
    @staticmethod
    def test_parse_date_valid():
        date_str = "2019-05-04T11:31:04.563Z"
        date_obj = utils.parse_date(date_str)

        assert isinstance(date_obj, datetime)

    @staticmethod
    def test_parse_date_no_data():
        date_str = None
        date_obj = utils.parse_date(date_str)

        assert date_obj is None

    @staticmethod
    def test_parse_int_valid():
        int_str = "2"
        int_number = utils.parse_int(int_str)

        assert isinstance(int_number, int)

    @staticmethod
    def test_parse_int_no_data():
        int_str = None
        int_number = utils.parse_int(int_str)

        assert int_number is None

    @staticmethod
    def test_parse_int_error():
        int_str = "string"
        int_number = utils.parse_int(int_str)

        assert int_number is None

    @staticmethod
    def test_parse_float_valid():
        float_str = "2.01"
        float_number = utils.parse_float(float_str)

        assert isinstance(float_number, float)

    @staticmethod
    def test_parse_float_no_data():
        float_str = None
        float_number = utils.parse_float(float_str)

        assert float_number is None

    @staticmethod
    def test_parse_float_error():
        float_str = "string"
        float_number = utils.parse_float(float_str)

        assert float_number is None

    @staticmethod
    def test_parse_bool_true_valid():
        bool_str = "1"
        bool_value = utils.parse_bool(bool_str)

        assert isinstance(bool_value, bool)

    @staticmethod
    def test_parse_bool_false_valid():
        bool_str = "0"
        bool_value = utils.parse_bool(bool_str)

        assert isinstance(bool_value, bool)

    @staticmethod
    def test_parse_bool_no_data():
        bool_str = None
        bool_value = utils.parse_bool(bool_str)

        assert bool_value is None

    @staticmethod
    def test_parse_bool_error():
        bool_str = "string"
        bool_value = utils.parse_bool(bool_str)

        assert bool_value is None



from datetime import datetime
from typing import List

from .grocy_api_client import GrocyApiClient, GrocyEntity
from .utils import parse_bool, parse_date, parse_float, parse_int

PRODUCTS_ENDPOINT = 'objects/products'


class Product(GrocyEntity):
    def __init__(self, parsed_json, api: GrocyApiClient):
        self.__id = parse_int(parsed_json.get('id'))
        self.__name = parsed_json.get('name')
        self.__description = parsed_json.get('description', None)
        self.__location_id = parse_int(parsed_json.get('location_id'))
        self.__qu_id_purchase = parse_int(parsed_json.get('qu_id_purchase'))
        self.__qu_id_stock = parse_int(parsed_json.get('qu_id_stock'))
        self.__enable_tare_weight_handling = parse_int(
            parsed_json.get('enable_tare_weight_handling', None))
        self.__not_check_stock_fulfillment_for_recipes = parse_int(
            parsed_json.get('not_check_stock_fulfillment_for_recipes'), None)
        self.__product_group_id = parse_int(
            parsed_json.get('product_group_id'), None)
        self.__qu_factor_purchase_to_stock = parse_float(
            parsed_json.get('qu_factor_purchase_to_stock'))
        self.__tare_weight = parse_float(parsed_json.get('tare_weight', None))
        barcodes_raw = parsed_json.get('barcode', "")
        if barcodes_raw:
            self.__barcodes = barcodes_raw.split(",")
        else:
            self.__barcodes = None
        self.__min_stock_amount = parse_int(
            parsed_json.get('min_stock_amount', None),
            0)
        self.__default_best_before_days = parse_int(
            parsed_json.get('default_best_before_days', None))
        self.__default_best_before_days_after_open = parse_int(
            parsed_json.get('default_best_before_days_after_open', None))
        self.__picture_file_name = parsed_json.get('picture_file_name', None)
        self.__allow_partial_units_in_stock = parse_bool(
            parsed_json.get('allow_partial_units_in_stock', False),
            False)
        self.__row_created_timestamp = parse_date(
            parsed_json.get('row_created_timestamp'))
        self.__endpoint = '{}/{}'.format(PRODUCTS_ENDPOINT, self.__id)
        super().__init__(api, self.__endpoint)

    @property
    def id(self) -> int:
        return self.__id

    @property
    def product_group_id(self) -> int:
        return self.__product_group_id

    @property
    def name(self) -> str:
        return self.__name

    @property
    def barcodes(self) -> List[str]:
        return self.__barcodes

    @property
    def location_id(self) -> int:
        return self.__location_id

    @property
    def qu_id_purchase(self) -> int:
        return self.__qu_id_purchase

    @property
    def description(self) -> str:
        return self.__description

    @property
    def qu_id_stock(self) -> int:
        return self.__qu_id_stock

    @property
    def enable_tare_weight_handling(self) -> int:
        return self.__enable_tare_weight_handling

    @property
    def not_check_stock_fulfillment_for_recipes(self) -> int:
        return self.__not_check_stock_fulfillment_for_recipes

    @property
    def qu_factor_purchase_to_stock(self) -> float:
        return self.__qu_factor_purchase_to_stock

    @property
    def tare_weight(self) -> float:
        return self.__tare_weight

    @property
    def min_stock_amount(self) -> int:
        return self.__min_stock_amount

    @property
    def default_best_before_days(self) -> int:
        return self.__default_best_before_days

    @property
    def default_best_before_days_after_open(self) -> int:
        return self.__default_best_before_days_after_open

    @property
    def picture_file_name(self) -> str:
        return self.__picture_file_name

    @property
    def allow_partial_units_in_stock(self) -> bool:
        return self.__allow_partial_units_in_stock

    @property
    def row_created_timestamp(self) -> datetime:
        return self.__row_created_timestamp



import json
from datetime import datetime
from test.test_const import CONST_BASE_URL, CONST_PORT, CONST_SSL
from unittest import TestCase

from pygrocydm.grocy_api_client import GrocyApiClient
from pygrocydm.quantity_unit import QUANTITY_UNITS_ENDPOINT, QuantityUnit


class TestQuantityUnit(TestCase):

    def setUp(self):
        self.api = GrocyApiClient(CONST_BASE_URL, "demo_mode",  verify_ssl=CONST_SSL, port=CONST_PORT)
        self.endpoint = QUANTITY_UNITS_ENDPOINT + '/2'

    def test_quantity_unit_data_diff_valid(self):
        quantity_unit = self.api.get_request(self.endpoint)
        quantity_unit_keys = quantity_unit.keys()
        moked_quantity_unit_json = """{
            "id": "2",
            "name": "Piece",
            "description": null,
            "row_created_timestamp": "2020-03-04 00:50:13",
            "name_plural": "Pieces",
            "plural_forms": null
        }"""
        moked_keys = json.loads(moked_quantity_unit_json).keys()
        self.assertCountEqual(list(quantity_unit_keys), list(moked_keys))

    def test_parse_json(self):
        quantity_unit = QuantityUnit(self.api.get_request(self.endpoint), self.api)
        assert isinstance(quantity_unit.id, int)
        assert isinstance(quantity_unit.description, str) or quantity_unit.description is None
        assert isinstance(quantity_unit.name, str)
        assert isinstance(quantity_unit.name_plural, str)
        assert isinstance(quantity_unit.plural_forms, str) or quantity_unit.plural_forms is None
        assert isinstance(quantity_unit.row_created_timestamp, datetime)



from datetime import datetime

from .grocy_api_client import GrocyApiClient, GrocyEntity
from .utils import parse_bool, parse_date, parse_float, parse_int

SHOPPING_LISTS_ENDPOINT = 'objects/shopping_lists'
SHOPPING_LIST_ENDPOINT = 'objects/shopping_list'


class ShoppingList(GrocyEntity):
    def __init__(self, parsed_json, api: GrocyApiClient):
        self.__id = parse_int(parsed_json.get('id'))
        self.__name = parsed_json.get('name')
        self.__description = parsed_json.get('description', None)
        self.__row_created_timestamp = parse_date(
            parsed_json.get('row_created_timestamp'))
        self.__endpoint = '{}/{}'.format(SHOPPING_LISTS_ENDPOINT, self.__id)
        super().__init__(api, self.__endpoint)

    @property
    def id(self) -> int:
        return self.__id

    @property
    def name(self) -> str:
        return self.__name

    @property
    def description(self) -> str:
        return self.__description

    @property
    def row_created_timestamp(self) -> datetime:
        return self.__row_created_timestamp


class ShoppingListItem(GrocyEntity):
    def __init__(self, parsed_json, api: GrocyApiClient):
        self.__id = parse_int(parsed_json.get('id'))
        self.__product_id = parse_int(parsed_json.get('product_id'), None)
        self.__note = parsed_json.get('note', None)
        self.__amount = parse_float(parsed_json.get('amount'), 0)
        self.__shopping_list_id = parse_int(
            parsed_json.get('shopping_list_id'))
        self.__done = parse_bool(parsed_json.get('done'), False)
        self.__row_created_timestamp = parse_date(
            parsed_json.get('row_created_timestamp'))
        self.__endpoint = '{}/{}'.format(SHOPPING_LIST_ENDPOINT, self.__id)
        super().__init__(api, self.__endpoint)

    @property
    def id(self) -> int:
        return self.__id

    @property
    def product_id(self) -> int:
        return self.__product_id

    @property
    def note(self) -> str:
        return self.__note

    @property
    def amount(self) -> float:
        return self.__amount

    @property
    def shopping_list_id(self) -> int:
        return self.__shopping_list_id

    @property
    def done(self) -> bool:
        return self.__done

    @property
    def row_created_timestamp(self) -> datetime:
        return self.__row_created_timestamp



from test.test_const import CONST_BASE_URL, CONST_PORT, CONST_SSL
from unittest import TestCase

import responses

from pygrocydm import GrocyDataManager
from pygrocydm.battery import Battery
from pygrocydm.chore import Chore
from pygrocydm.location import Location
from pygrocydm.product import Product
from pygrocydm.quantity_unit import QuantityUnit
from pygrocydm.shopping_list import ShoppingList, ShoppingListItem


class TestGrocyDataManager(TestCase):

    def setUp(self):
        self.gdm = GrocyDataManager(CONST_BASE_URL, "api_key")
        self.gdm = None
        self.gdm = GrocyDataManager(CONST_BASE_URL, "demo_mode",  verify_ssl = CONST_SSL, port = CONST_PORT)

    def test_init(self):
        assert isinstance(self.gdm, GrocyDataManager)

    def test_products_valid(self):
        products = self.gdm.products().list
        assert isinstance(products, tuple)
        assert len(products) >=1
        for product in products:
            assert isinstance(product, Product)

    @responses.activate
    def test_products_invalid_no_data(self):
        resp = []
        responses.add(responses.GET,
            '{}:{}/api/objects/products'.format(CONST_BASE_URL, CONST_PORT),
            json=resp,
            status=200)
        products = self.gdm.products().list
        assert products is None

    @responses.activate
    def test_products_error(self):
        responses.add(responses.GET,
            '{}:{}/api/objects/products'.format(CONST_BASE_URL, CONST_PORT),
            status=400)
        products = self.gdm.products().list
        assert products is None

    def test_chores_valid(self):
        chores = self.gdm.chores().list
        assert isinstance(chores, tuple)
        assert len(chores) >=1
        for chore in chores:
            assert isinstance(chore, Chore)

    def test_locations_valid(self):
        locations = self.gdm.locations().list
        assert isinstance(locations, tuple)
        assert len(locations) >=1
        for location in locations:
            assert isinstance(location, Location)

    def test_batteries_valid(self):
        batteries = self.gdm.batteries().list
        assert isinstance(batteries, tuple)
        assert len(batteries) >=1
        for battery in batteries:
            assert isinstance(battery, Battery)

    def test_shopping_list_items_valid(self):
        shopping_list_items = self.gdm.shopping_list().list
        assert isinstance(shopping_list_items, tuple)
        assert len(shopping_list_items) >=1
        for items in shopping_list_items:
            assert isinstance(items, ShoppingListItem)

    def test_shopping_lists_valid(self):
        shopping_lists = self.gdm.shopping_lists().list
        assert isinstance(shopping_lists, tuple)
        assert len(shopping_lists) >=1
        for shopping_list in shopping_lists:
            assert isinstance(shopping_list, ShoppingList)

    def test_add_product_valid(self):
        product_list = self.gdm.products()
        old_len = len(product_list.list)
        new_product = {}
        new_product['name'] = 'Test product'
        new_product['location_id'] = 1
        new_product['qu_id_purchase'] = 1
        new_product['qu_id_stock'] = 1
        new_product['qu_factor_purchase_to_stock'] = 1
        new_product_id = product_list.add(new_product)
        new_len = len(product_list.list)
        assert isinstance(new_product_id, int)
        assert new_len == old_len + 1

    def test_add_product_error(self):
        product_list = self.gdm.products()
        new_product = {}
        new_product['name'] = 'Test product'
        resp = product_list.add(new_product)
        assert "error_message" in resp.text

    def test_edit_product_valid(self):
        fields = {}
        fields['name'] = 'Test'
        assert self.gdm.products().list[-1].edit(fields)

    def test_edit_product_error(self):
        fields = {}
        fields['nam'] = 'Test'
        assert "error_message" in self.gdm.products().list[0].edit(fields)

    def test_product_delete_valid(self):
        product = self.gdm.products().list[-1]
        old_len = len(self.gdm.products().list)
        assert product.delete()
        self.gdm.products().refresh()
        new_len = len(self.gdm.products().list)
        assert new_len == old_len - 1

    def test_product_delete_error(self):
        product = self.gdm.products().list[-1]
        product.delete()
        assert 'error' in product.delete()

    def test_search_product_valid(self):
        products = self.gdm.products().search("Co")
        assert isinstance(products, tuple)
        assert len(products) >=1
        for product in products:
            assert isinstance(product, Product)

    def test_search_product_invalid_no_data(self):
        products = self.gdm.products().search("Toto")
        assert products is None

    @responses.activate
    def test_search_product_error(self):
        url = '{}:{}/api/objects/products'.format(CONST_BASE_URL, CONST_PORT)
        responses.add_passthru(url)
        responses.add(responses.GET,
            '{}/search/error'.format(url),
            status=400)
        products = self.gdm.products().search("error")
        assert products is None

    def test_quantity_units_valid(self):
        quantity_units = self.gdm.quantity_units().list
        assert isinstance(quantity_units, tuple)
        assert len(quantity_units) >=1
        for quantity_unit in quantity_units:
            assert isinstance(quantity_unit, QuantityUnit)



from datetime import datetime
from enum import Enum

from .grocy_api_client import GrocyApiClient, GrocyEntity
from .utils import parse_bool, parse_date, parse_int

CHORES_ENDPOINT = 'objects/chores'


class PeriodType(Enum):
    MANUALLY = "manually"
    DYNAMIC_REGULAR = "dynamic-regular"
    DAILY = "daily"
    WEEKLY = "weekly"
    MONTHLY = "monthly"


class AssignmentType(Enum):
    NO_ASSIGNMENT = "no-assignment"
    WHO_LEAST_DID_FIRST = "who-least-did-first"
    RANDOM = "random"
    IN_ALPHABETICAL_ORDER = "in-alphabetical-order"


class Chore(GrocyEntity):
    def __init__(self, parsed_json, api: GrocyApiClient):
        self.__id = parse_int(parsed_json.get('id'))
        self.__name = parsed_json.get('name')
        self.__description = parsed_json.get('description', None)
        self.__period_type = parsed_json.get('period_type')
        self.__period_config = parsed_json.get('period_config', None)
        self.__period_days = parse_int(parsed_json.get('period_days'), None)
        self.__track_date_only = parse_bool(
            parsed_json.get('track_date_only'), False)
        self.__rollover = parse_bool(parsed_json.get('rollover'), False)
        self.__assignment_type = parsed_json.get('assignment_type', None)
        self.__assignment_config = parsed_json.get('assignment_config', None)
        self.__next_execution_assigned_to_user_id = parse_int(
            parsed_json.get('next_execution_assigned_to_user_id'),
            None)
        self.__row_created_timestamp = parse_date(
            parsed_json.get('row_created_timestamp'))
        self.__endpoint = '{}/{}'.format(CHORES_ENDPOINT, self.__id)
        super().__init__(api, self.__endpoint)

    @property
    def id(self) -> int:
        return self.__id

    @property
    def name(self) -> str:
        return self.__name

    @property
    def description(self) -> str:
        return self.__description

    @property
    def period_type(self) -> str:
        return self.__period_type

    @property
    def period_config(self) -> str:
        return self.__period_config

    @property
    def period_days(self) -> int:
        return self.__period_days

    @property
    def track_date_only(self) -> bool:
        return self.__track_date_only

    @property
    def rollover(self) -> bool:
        return self.__rollover

    @property
    def assignment_type(self) -> str:
        return self.__assignment_type

    @property
    def assignment_config(self) -> str:
        return self.__assignment_config

    @property
    def next_execution_assigned_to_user_id(self) -> int:
        return self.__next_execution_assigned_to_user_id

    @property
    def row_created_timestamp(self) -> datetime:
        return self.__row_created_timestamp



import iso8601


def parse_date(input_value):
    if input_value is None:
        return None
    return iso8601.parse_date(input_value)


def parse_int(input_value, default_value=None):
    if input_value is None:
        return default_value
    try:
        return int(input_value)
    except ValueError:
        return default_value


def parse_float(input_value, default_value=None):
    if input_value is None:
        return default_value
    try:
        return float(input_value)
    except ValueError:
        return default_value


def parse_bool(input_value, default_value=None):
    if input_value is None:
        return default_value
    if str(input_value) == '1' or input_value == "True":
        return True
    if str(input_value) == '0' or input_value == "False":
        return False
    return default_value



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pygrocydm",
    version="0.3.0",
    author="Blueblueblob",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/BlueBlueBlob/pygrocydm",
    license='MIT',
    packages=setuptools.find_packages(),
    install_requires=[
        "requests",
        "iso8601",
        "pytz",
        "tzlocal"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import json
from datetime import datetime
from test.test_const import CONST_BASE_URL, CONST_PORT, CONST_SSL
from unittest import TestCase

from pygrocydm.grocy_api_client import GrocyApiClient
from pygrocydm.product import Product


class TestProduct(TestCase):

    def setUp(self):
        self.api = GrocyApiClient(CONST_BASE_URL, "demo_mode",  verify_ssl=CONST_SSL, port=CONST_PORT)
        self.endpoint = 'objects/products/1'

    def test_product_data_diff_valid(self):
        product = self.api.get_request(self.endpoint)
        product_keys = product.keys()
        moked_product_json = """{
            "id": "1",
            "name": "Cookies",
            "description": null,
            "location_id": "4",
            "qu_id_purchase": "3",
            "qu_id_stock": "3",
            "qu_factor_purchase_to_stock": "1.0",
            "barcode": null,
            "min_stock_amount": "8",
            "default_best_before_days": "0",
            "row_created_timestamp": "2020-02-25 00:50:13",
            "product_group_id": "1",
            "picture_file_name": "cookies.jpg",
            "default_best_before_days_after_open": "0",
            "allow_partial_units_in_stock": "0",
            "enable_tare_weight_handling": "0",
            "tare_weight": "0.0",
            "not_check_stock_fulfillment_for_recipes": "0",
            "parent_product_id": null,
            "calories": "123",
            "cumulate_min_stock_amount_of_sub_products": "0",
            "default_best_before_days_after_freezing": "0",
            "default_best_before_days_after_thawing": "0"
        }"""
        moked_keys = json.loads(moked_product_json).keys()
        self.assertCountEqual(list(product_keys), list(moked_keys))

    def test_parse_json(self):
        product = Product(self.api.get_request(self.endpoint), self.api)
        assert isinstance(product.id, int)
        assert isinstance(product.product_group_id, int)
        assert isinstance(product.name, str)
        assert isinstance(product.barcodes, (str, list)) or product.barcodes is None
        assert isinstance(product.location_id, int) or product.location_id is None
        assert isinstance(product.qu_id_purchase, int) or product.qu_id_purchase is None
        assert isinstance(product.description, str) or product.description is None
        assert isinstance(product.qu_id_stock, int) or product.qu_id_stock is None
        assert isinstance(product.enable_tare_weight_handling, int) or product.enable_tare_weight_handling is None
        assert isinstance(product.not_check_stock_fulfillment_for_recipes, int) or product.not_check_stock_fulfillment_for_recipes is None
        assert isinstance(product.qu_factor_purchase_to_stock, float) or product.qu_factor_purchase_to_stock is None
        assert isinstance(product.tare_weight, float) or product.tare_weight is None
        assert isinstance(product.min_stock_amount, int) or product.min_stock_amount == 0
        assert isinstance(product.default_best_before_days, int) or product.default_best_before_days is None
        assert isinstance(product.default_best_before_days_after_open, int) or product.default_best_before_days_after_open is None
        assert isinstance(product.picture_file_name, str) or product.picture_file_name is None
        assert isinstance(product.allow_partial_units_in_stock, bool)
        assert isinstance(product.row_created_timestamp, datetime)



import json
from datetime import datetime
from test.test_const import CONST_BASE_URL, CONST_PORT, CONST_SSL
from unittest import TestCase

from pygrocydm.chore import CHORES_ENDPOINT, Chore
from pygrocydm.grocy_api_client import GrocyApiClient


class TestChore(TestCase):

    def setUp(self):
        self.api = GrocyApiClient(CONST_BASE_URL, "demo_mode",  verify_ssl=CONST_SSL, port=CONST_PORT)
        self.endpoint = CHORES_ENDPOINT + '/1'

    def test_chore_data_diff_valid(self):
        chore = self.api.get_request(self.endpoint)
        chore_keys = chore.keys()
        moked_chore_json = """{
            "id": "1",
            "name": "Changed towels in the bathroom",
            "description": null,
            "period_type": "manually",
            "period_days": "5",
            "row_created_timestamp": "2020-02-26 00:50:11",
            "period_config": null,
            "track_date_only": "0",
            "rollover": "0",
            "assignment_type": null,
            "assignment_config": null,
            "next_execution_assigned_to_user_id": null,
            "consume_product_on_execution": "0",
            "product_id": null,
            "product_amount": null,
            "period_interval": "1"
        }"""
        moked_keys = json.loads(moked_chore_json).keys()
        self.assertCountEqual(list(chore_keys), list(moked_keys))

    def test_parse_json(self):
        chore = Chore(self.api.get_request(self.endpoint), self.api)
        assert isinstance(chore.id, int)
        assert isinstance(chore.description, str) or chore.description is None
        assert isinstance(chore.name, str)
        assert isinstance(chore.period_type, str)
        assert isinstance(chore.period_days, int) or chore.period_days is None
        assert isinstance(chore.next_execution_assigned_to_user_id, int) or chore.next_execution_assigned_to_user_id is None
        assert isinstance(chore.description, str) or chore.description is None
        assert isinstance(chore.period_config, str) or chore.period_config is None
        assert isinstance(chore.track_date_only, bool) or not chore.track_date_only
        assert isinstance(chore.rollover, bool) or not chore.rollover
        assert isinstance(chore.assignment_type, str) or chore.assignment_type is None
        assert isinstance(chore.assignment_config, str) or chore.assignment_config is None
        assert isinstance(chore.row_created_timestamp, datetime)



import json
from datetime import datetime
from test.test_const import CONST_BASE_URL, CONST_PORT, CONST_SSL
from unittest import TestCase

from pygrocydm.grocy_api_client import GrocyApiClient
from pygrocydm.shopping_list import (SHOPPING_LIST_ENDPOINT,
                                     SHOPPING_LISTS_ENDPOINT, ShoppingList,
                                     ShoppingListItem)


class TestShoppingList(TestCase):

    def setUp(self):
        self.api = GrocyApiClient(CONST_BASE_URL, "demo_mode",  verify_ssl=CONST_SSL, port=CONST_PORT)
        self.endpoint = SHOPPING_LISTS_ENDPOINT + '/1'

    def test_shopping_list_data_diff_valid(self):
        shopping_list = self.api.get_request(self.endpoint)
        shopping_list_keys = shopping_list.keys()
        moked_shopping_list_json = """{
            "id": "1",
            "name": "Shopping list",
            "description": null,
            "row_created_timestamp": "2020-03-02 00:50:09"
        }"""
        moked_keys = json.loads(moked_shopping_list_json).keys()
        self.assertCountEqual(list(shopping_list_keys), list(moked_keys))

    def test_parse_json(self):
        shopping_list = ShoppingList(self.api.get_request(self.endpoint), self.api)
        assert isinstance(shopping_list.id, int)
        assert isinstance(shopping_list.description, str) or shopping_list.description is None
        assert isinstance(shopping_list.name, str)
        assert isinstance(shopping_list.row_created_timestamp, datetime)


class TestShoppingListItem(TestCase):

    def setUp(self):
        self.api = GrocyApiClient(CONST_BASE_URL, "demo_mode",  verify_ssl=CONST_SSL, port=CONST_PORT)
        self.endpoint = SHOPPING_LIST_ENDPOINT + '/1'

    def test_shopping_list_item_data_diff_valid(self):
        shopping_list_item = self.api.get_request(self.endpoint)
        shopping_list_item_keys = shopping_list_item.keys()
        moked_shopping_list_item_json = """{
            "id": "1",
            "product_id": null,
            "note": "Some good snacks",
            "amount": "1",
            "row_created_timestamp": "2020-03-02 00:50:10",
            "shopping_list_id": "1",
            "done": "0"
        }"""
        moked_keys = json.loads(moked_shopping_list_item_json).keys()
        self.assertCountEqual(list(shopping_list_item_keys), list(moked_keys))

    def test_parse_json(self):
        shopping_list_item = ShoppingListItem(self.api.get_request(self.endpoint), self.api)
        assert isinstance(shopping_list_item.id, int)
        assert isinstance(shopping_list_item.product_id, int) or shopping_list_item.product_id is None
        assert isinstance(shopping_list_item.note, str) or shopping_list_item.note is None
        assert isinstance(shopping_list_item.amount, float)
        assert isinstance(shopping_list_item.shopping_list_id, int)
        assert isinstance(shopping_list_item.done, bool)
        assert isinstance(shopping_list_item.row_created_timestamp, datetime)



from .battery import BATTERIES_ENDPOINT, Battery
from .chore import CHORES_ENDPOINT, Chore
from .grocy_api_client import (DEFAULT_PORT_NUMBER, GrocyApiClient,
                               GrocyEntityList)
from .location import LOCATION_ENDPOINT, Location
from .product import PRODUCTS_ENDPOINT, Product
from .quantity_unit import QUANTITY_UNITS_ENDPOINT, QuantityUnit
from .shopping_list import (SHOPPING_LIST_ENDPOINT, SHOPPING_LISTS_ENDPOINT,
                            ShoppingList, ShoppingListItem)


class GrocyDataManager():
    def __init__(
            self, base_url, api_key,
            port: int = DEFAULT_PORT_NUMBER,
            verify_ssl=True):
        self.__api = GrocyApiClient(base_url, api_key, port, verify_ssl)

    def products(self) -> GrocyEntityList:
        cls = Product
        return GrocyEntityList(self.__api, cls, PRODUCTS_ENDPOINT)

    def chores(self) -> GrocyEntityList:
        cls = Chore
        return GrocyEntityList(self.__api, cls, CHORES_ENDPOINT)

    def locations(self) -> GrocyEntityList:
        cls = Location
        return GrocyEntityList(self.__api, cls, LOCATION_ENDPOINT)

    def batteries(self) -> GrocyEntityList:
        cls = Battery
        return GrocyEntityList(self.__api, cls, BATTERIES_ENDPOINT)

    def shopping_list(self) -> GrocyEntityList:
        cls = ShoppingListItem
        return GrocyEntityList(self.__api, cls, SHOPPING_LIST_ENDPOINT)

    def shopping_lists(self) -> GrocyEntityList:
        cls = ShoppingList
        return GrocyEntityList(self.__api, cls, SHOPPING_LISTS_ENDPOINT)

    def quantity_units(self) -> GrocyEntityList:
        cls = QuantityUnit
        return GrocyEntityList(self.__api, cls, QUANTITY_UNITS_ENDPOINT)
